@@ -1,5 +1,7 @@
 
 
+import numpy as np
+
 
 # #############################################################################
 # #############################################################################
@@ -109,47 +111,47 @@ def get_fupdate(handle=None, dtype=None, norm=None, bstr=None):
         func = lambda val, handle=handle: handle.set_ydata(val)
     elif dtype in ['data']:   # Also works for imshow
         func = lambda val, handle=handle: handle.set_data(val)
+    elif dtype in ['data.T']:   # Also works for imshow
+        func = lambda val, handle=handle: handle.set_data(val.T)
     elif dtype in ['alpha']:   # Also works for imshow
         func = lambda val, handle=handle, norm=norm: handle.set_alpha(norm(val))
     elif dtype == 'txt':
         func = lambda val, handle=handle, bstr=bstr: handle.set_text(bstr.format(val))
+    else:
+        msg = f'Unknown mobile dtype: {dtype}'
+        raise Exception(msg)
     return func
 
 
-def _update_mobile_data(
-    func=None,
-    kref=None,
-    kdata=None,
-    iref=None,
-    ddata=None,
-):
-    """"""
+def _get_slice(laxis=None, ndim=None):
 
-    # if handle.__class__.__name__ == 'Line2D':
+    nax = len(laxis)
+    assert nax in range(1, ndim + 1)
 
-    if kdata == 'index':
-        func(iref)
+    if ndim == nax:
+        def fslice(*args):
+            return args
 
-    elif ddata[kdata]['data'].ndim == 1:
-        func(ddata[kdata]['data'][iref])
+    else:
+        def fslice(*args, laxis=laxis):
+            ind = [slice(None) for ii in range(ndim)]
+            for ii, aa in enumerate(args):
+                ind[laxis[ii]] = aa
+            return tuple(ind)
 
-    elif ddata[kdata]['data'].ndim == 2:
+    return fslice
 
-        idim = ddata[kdata]['ref'].index(kref)
-        if idim == 0:
-            func(ddata[kdata]['data'][iref, :])
-        else:
-            func(ddata[kdata]['data'][:, iref])
 
-    elif ddata[kdata]['data'].ndim == 3:
+def get_slice(nocc=None, laxis=None, lndim=None):
 
-        idim = ddata[kdata]['ref'].index(kref)
-        if idim == 0:
-            func(ddata[kdata]['data'][iref, :, :])
-        elif idim == 1:
-            func(ddata[kdata]['data'][:, iref, :])
-        elif idim == 2:
-            func(ddata[kdata]['data'][:, :, iref])
+    if nocc == 1:
+        return [_get_slice(laxis=laxis, ndim=lndim[0])]
+
+    elif nocc == 2:
+        return [
+            _get_slice(laxis=[laxis[0]], ndim=lndim[0]),
+            _get_slice(laxis=[laxis[1]], ndim=lndim[1]),
+        ]
 
 
 def _update_mobile(k0=None, dmobile=None, dref=None, ddata=None):
@@ -158,22 +160,42 @@ def _update_mobile(k0=None, dmobile=None, dref=None, ddata=None):
     func = dmobile[k0]['func']
     kref = dmobile[k0]['ref']
     kdata = dmobile[k0]['data']
-    iref = [dref[rr]['indices'][dmobile[k0]['ind']] for rr in kref]
 
-    if kref[0] is not None:
-        _update_mobile_data(
-            func=func[0],
-            kref=kref[0],
-            kdata=kdata[0],
-            iref=iref[0],
-            ddata=ddata,
-        )
+    # All ref do not necessarily have the same nb of indices
+    iref = [
+        dref[rr]['indices'][
+            min(dmobile[k0]['ind'], len(dref[rr]['indices']) - 1)
+        ]
+        for rr in dmobile[k0]['ref']
+    ]
 
-    if len(kref) > 1 and kref[1] is not None:
-        _update_mobile_data(
-            func=func[1],
-            kref=kref[1],
-            kdata=kdata[1],
-            iref=iref[1],
-            ddata=ddata,
+    nocc = len(set(dmobile[k0]['dtype']))
+    if nocc == 1:
+        c0 = (
+            dmobile[k0]['data'][0] == 'index'
+            or ddata[dmobile[k0]['data'][0]]['data'].dtype.type == np.str_
         )
+        if c0:
+            dmobile[k0]['func_set_data'][0](*iref)
+
+        else:
+            dmobile[k0]['func_set_data'][0](
+                ddata[dmobile[k0]['data'][0]]['data'][
+                    dmobile[k0]['func_slice'][0](*iref)
+                ]
+            )
+
+    else:
+        for ii in range(nocc):
+            c0 = (
+                dmobile[k0]['data'][0] == 'index'
+                or ddata[dmobile[k0]['data'][0]]['data'].dtype.type == np.str_
+            )
+            if c0:
+                dmobile[k0]['func_set_data'][ii](iref[ii])
+            else:
+                dmobile[k0]['func_set_data'][ii](
+                    ddata[dmobile[k0]['data'][ii]]['data'][
+                        dmobile[k0]['func_slice'][ii](iref[ii])
+                    ]
+                )
