@@ -16,6 +16,9 @@ class DataStock3(DataStock2):
         self,
         # parameters
         key=None,
+        keyX=None,
+        keyY=None,
+        keyZ=None,
         ind=None,
         vmin=None,
         vmax=None,
@@ -26,6 +29,9 @@ class DataStock3(DataStock2):
         dinc=None,
         lkeys=None,
         bstr_dict=None,
+        rotation=None,
+        inverty=None,
+        bck=None,
         # figure-specific
         dax=None,
         dmargin=None,
@@ -40,6 +46,9 @@ class DataStock3(DataStock2):
             # parameters
             coll=self,
             key=key,
+            keyX=keyX,
+            keyY=keyY,
+            keyZ=keyZ,
             ind=ind,
             vmin=vmin,
             vmax=vmax,
@@ -50,6 +59,9 @@ class DataStock3(DataStock2):
             dinc=dinc,
             lkeys=lkeys,
             bstr_dict=bstr_dict,
+            rotation=rotation,
+            inverty=inverty,
+            bck=bck,
             # figure-specific
             dax=dax,
             dmargin=dmargin,
@@ -65,6 +77,8 @@ class DataStock3(DataStock2):
         # parameters
         keyA=None,
         keyB=None,
+        keyX=None,
+        axis=None,
         # customization of scatter plot
         dlim=None,
         color_dict=None,
@@ -82,6 +96,8 @@ class DataStock3(DataStock2):
         dist_cmap=None,
         dist_min=None,
         dist_max=None,
+        dist_sample_min=None,
+        dist_rel=None,
         # customization of interactivity
         ind0=None,
         nmax=None,
@@ -102,6 +118,8 @@ class DataStock3(DataStock2):
             coll=self,
             keyA=keyA,
             keyB=keyB,
+            keyX=keyX,
+            axis=axis,
             # customization of scatter plot
             dlim=dlim,
             color_dict=color_dict,
@@ -119,6 +137,8 @@ class DataStock3(DataStock2):
             dist_cmap=dist_cmap,
             dist_min=dist_min,
             dist_max=dist_max,
+            dist_sample_min=dist_sample_min,
+            dist_rel=dist_rel,
             # customization of interactivity
             ind0=ind0,
             nmax=nmax,
